@@ -57,6 +57,9 @@ if args.has_key("context"):
 # Test the store.
 
 if not args.has_key("no-test"):
+
+    print "Test addition, literal/string equivalence, removal."
+
     s.add((u"a", u"b", u""))
     s.add((u"a", u"b", Literal(u""))) # should have no effect
     s.add(("a", "b", "x"))
@@ -69,6 +72,10 @@ if not args.has_key("no-test"):
     print "Should be 0 triples:", len(result) == 0
     print result
     print
+
+    print "--------"
+    print "Test addition, matching, ordering, expressions, expressions plus ordering."
+
     s.add(("p", "q", Literal("20050101")))
     s.add(("p", "q", Literal("20050401")))
     s.add(("p", "q", Literal("20051001")))
@@ -77,20 +84,23 @@ if not args.has_key("no-test"):
     print "Should be 4 triples:", len(result) == 4
     print result
     print
-    result = s.triples(("p", "q", None), ordering=["desc"])
+    result = s.triples(("p", "q", None), ordering="desc")
     print "Should be 4 triples (in descending order):", len(result) == 4
     print result
     print
-    # Test of expression as object.
     result = s.triples(("p", "q", ("_ > ? and _ < ?", ["20050301", "20051101"])))
     print "Should be 2 triples:", len(result) == 2
     print result
     print
-    # Test of expression as object.
-    result = s.triples(("p", "q", ("_ > ? and _ < ?", ["20050301", "20051101"])), ordering=["q", "asc"])
+    result = s.triples(("p", "q", ("_ > ? and _ < ?", ["20050301", "20051101"])), ordering="asc")
     print "Should be 2 triples (in order):", len(result) == 2
     print result
     print
+
+    print "--------"
+    print "Test subjects, addition and deletion using subjects, chaining, and"
+    print "widening."
+
     subject = s.subject("p")
     result = subject["q"]
     print "Should be 4 results:", len(result) == 4
@@ -102,11 +112,26 @@ if not args.has_key("no-test"):
     print "Should be 4 results:", len(result) == 4
     print result
     print
+    s.add((URIRef("N1"), URIRef("O1"), "p"))
+    s.add((URIRef("N2"), URIRef("O2"), "q"))
+    result = s.triples((None, None, subject))
+    print "Should be 1 result:", len(result) == 1
+    print result
+    print
+    result = s.triples((subject, None, None))
+    print "Should be 4 results:", len(result) == 4
+    print result
+    print
+    s.remove((None, [URIRef("O1"), URIRef("O2")], None))
     del subject["q"]
     result = subject["q"]
     print "Should be 0 results:", len(result) == 0
     print result
     print
+
+    print "--------"
+    print "Test BNode creation and deletion."
+
     s.add((URIRef("xyz"), URIRef("abc"), BNode()))
     s.add((URIRef("xyz"), URIRef("abc"), BNode()))
     result = s.triples((URIRef("xyz"), None, None))
@@ -115,6 +140,10 @@ if not args.has_key("no-test"):
     print result
     print
     s.remove((URIRef("xyz"), None, None))
+
+    print "--------"
+    print "Test range comparisons, predicates, usage of predicates in patterns."
+
     s.add((URIRef("a"), URIRef("b"), Literal("1")))
     s.add((URIRef("a"), URIRef("b"), Literal("2")))
     s.add((URIRef("a"), URIRef("b"), Literal("3")))
@@ -124,15 +153,14 @@ if not args.has_key("no-test"):
     s.add((URIRef("b"), URIRef("x"), Literal("3")))
     s.add((URIRef("b"), URIRef("x"), Literal("4")))
     s.add((URIRef("b"), URIRef("x"), Literal("5")))
-    # Test of expression as object.
     t1 = s.triples((None, None, ("_ > ?", Literal("3"))))
     t2 = s.triples((None, URIRef("x"), None))
-    result = s.triples([t1, t2])
+    result = s.and_triples([t1, t2])
     print "Should be 2 triples:", len(result) == 2
     print result
     print
     p1 = s.predicates(URIRef("a"))
-    result = s.triples([p1, t1, t2])
+    result = s.and_triples([p1, t1, t2])
     print "Should be 2 triples:", len(result) == 2
     print result
     print
@@ -145,89 +173,172 @@ if not args.has_key("no-test"):
     print "Should be 6 triples:", len(t2) == 6
     print t2
     print
-    s1 = s.subjects(URIRef("b"))
-    print "Should be 1 result:", len(s1) == 1
-    print s1
+
+    print "--------"
+    print "Test subjects, subjects plus expressions."
+
+    subjects_with_predicate_b = s.subjects(URIRef("b"))
+    print "Should be 1 result:", len(subjects_with_predicate_b) == 1
+    print subjects_with_predicate_b
     print
-    # Test of expression as object.
-    s2 = s.subjects(None, ("_ > ?", [Literal("2")]))
-    print "Should be 2 results:", len(s2) == 2
-    print s2
+    subjects_with_object_gt_2 = s.subjects(None, ("_ > ?", [Literal("2")]))
+    print "Should be 2 results:", len(subjects_with_object_gt_2) == 2
+    print subjects_with_object_gt_2
     print
-    # Test of expression as object plus negation.
-    # NOTE: Since negation wraps existing results and doesn't offer a proper query,
-    # NOTE: we must wrap it in another call.
-    not_s2 = s.triples([s.negation(s.subjects(None, ("_ > ?", [Literal("2")])))])
-    print "Should be 0 results:", len(not_s2) == 0
-    print not_s2
+
+    print "--------"
+    print "Test negations of subjects."
+
+    triples_for_subjects_not_with_predicate_b = s.not_triples(subjects_with_predicate_b)
+    print "Should be 6 results:", len(triples_for_subjects_not_with_predicate_b) == 3
+    print triples_for_subjects_not_with_predicate_b
     print
-    # Test of presence.
+
+    print "--------"
+    print "Test of expression as object plus negation."
+
+    triples_for_subjects_not_with_object_gt_2 = s.not_triples(s.subjects(None, ("_ > ?", [Literal("2")])))
+    print "Should be 0 results:", len(triples_for_subjects_not_with_object_gt_2) == 0
+    print triples_for_subjects_not_with_object_gt_2
+    print
+
+    print "--------"
+    print "Test of presence."
+
     has_b = s.triples((None, URIRef("b"), s.Defined()))
     print "Should be 3 results:", len(has_b) == 3
     print has_b
     print
-    # Test of absence.
-    # NOTE: Since negation wraps existing results and doesn't offer a proper query,
-    # NOTE: we must wrap it in another call.
-    not_has_b = s.triples([s.negation(s.triples((None, URIRef("b"), s.Defined())))])
+
+    print "--------"
+    print "Test of absence."
+
+    not_has_b = s.not_triples(s.triples((None, URIRef("b"), s.Defined())))
     print "Should be 6 results:", len(not_has_b) == 6
     print not_has_b
     print
-    # Test of triple combination.
-    s1_and_s2 = s.triples([s1, s2])
-    print "Should be 6 triples:", len(s1_and_s2) == 6
-    print s1_and_s2
+
+    print "--------"
+    print "Test of subject combination."
+
+    triples_for_subjects_b_obj_gt_2 = s.triples(expression=s.and_subjects([subjects_with_predicate_b, subjects_with_object_gt_2]))
+    print "Should be 6 triples:", len(triples_for_subjects_b_obj_gt_2) == 6
+    print triples_for_subjects_b_obj_gt_2
     print
-    # Test of triple combination using intersections.
-    s1_and_s2 = s.intersection([s1, s2])
-    print "Should be 1 triple:", len(s1_and_s2) == 1
-    print s1_and_s2
+
+    print "--------"
+    print "Test of triple combination."
+
+    triples_for_subjects_b_obj_gt_2 = s.and_triples([subjects_with_predicate_b, subjects_with_object_gt_2])
+    print "Should be 6 triples:", len(triples_for_subjects_b_obj_gt_2) == 6
+    print triples_for_subjects_b_obj_gt_2
     print
-    # Test of triple combination using conjunctions and negations.
-    not_s1_and_s2 = s.triples(s.Conjunction([s.negation(s1), s2]))
-    print "Should be 3 triples:", len(not_s1_and_s2) == 3
-    print not_s1_and_s2
+
+    print "--------"
+    print "Test of triple combination using intersections."
+
+    subjects_b_obj_gt_2 = s.intersect_subjects([subjects_with_predicate_b, subjects_with_object_gt_2])
+    print "Should be 1 subject:", len(subjects_b_obj_gt_2) == 1
+    print subjects_b_obj_gt_2
     print
-    # Test of triple combination.
-    s1_and_s2 = s.triples([s1, s2], ordering=["asc"])
-    print "Should be 6 triples (in order):", len(s1_and_s2) == 6
-    print s1_and_s2
+
+    print "--------"
+    print "Test of triple combination using conjunctions and negations."
+
+    triples_with_predicate_x_obj_gt_2 = s.and_triples([s.not_triples(subjects_with_predicate_b), subjects_with_object_gt_2])
+    print "Should be 3 triples:", len(triples_with_predicate_x_obj_gt_2) == 3
+    print triples_with_predicate_x_obj_gt_2
     print
-    # Test of triple combination.
-    s1_and_s2_x = s.triples([s1, s2], ordering=[URIRef("x"), "asc"])
-    print "Should be 3 triples (in order):", len(s1_and_s2_x) == 3
-    print s1_and_s2_x
+
+    print "--------"
+    print "Test of triple combination."
+
+    triples_a = ordered_triples_b_obj_gt_2 = s.triples(expression=s.and_triples([subjects_with_predicate_b, subjects_with_object_gt_2]), ordering="asc")
+    print "Should be 6 triples (in order):", len(ordered_triples_b_obj_gt_2) == 6
+    print ordered_triples_b_obj_gt_2
     print
+
+    print "--------"
+    print "Test of triple combination."
+
+    ordered_triples_x_gt_2 = s.triples((None, URIRef("x"), None),
+        expression=s.and_triples([subjects_with_predicate_b, subjects_with_object_gt_2]), ordering="asc")
+    print "Should be 3 triples (in order):", len(ordered_triples_x_gt_2) == 3
+    print ordered_triples_x_gt_2
+    print
+
+    print "--------"
+    print "Test of tuples and ordering."
+
     s.add((URIRef("c"), URIRef("p"), URIRef("a")))
     s.add((URIRef("c"), URIRef("q"), URIRef("a")))
     s.add((URIRef("d"), URIRef("p"), URIRef("a")))
     s.add((URIRef("d"), URIRef("p"), URIRef("b")))
-    ordered = s.triples((None, None, None), ordering=[URIRef("p"), URIRef("x"), "asc"])
-    print "Should be 9 triples (in order):", len(ordered) == 9
+    ordered = s.tuples((None, URIRef("p"), URIRef("x"), None), ordering="asc")
+    print "Should be 8 tuples (in order):", len(ordered) == 8
     print ordered
     print
-    # Test of triples as object.
-    s1_and_s2_p_x = s.triples((None, URIRef("p"), s1_and_s2), ordering=[URIRef("p"), URIRef("x"), "asc"])
-    print "Should be 6 triples (in order):", len(s1_and_s2_p_x) == 6
-    print s1_and_s2_p_x
+
+    print "--------"
+    print "Test of tuples and chaining."
+
+    s.add((URIRef("B"), URIRef("C"), URIRef("c")))
+    s.add((URIRef("c"), URIRef("E"), URIRef("d")))
+    chained = s.tuples((None, URIRef("C"), URIRef("E"), ordered))
+    print "Should be 1 tuple:", len(chained) == 1
+    print chained
     print
+    extended = s.tuples(chained.pattern[:-1] + ordered.pattern[1:])
+    print "Should be 5 tuples:", len(extended) == 5
+    print extended
+    print
+
+    print "--------"
+    print "Test of negation with tuples."
+
+    all_tuples = s.tuples((None, None, None, None))
+    negated = s.not_tuples(chained, (None, None, None, None))
+    print "Should be all but one of the tuples:", len(negated) == len(all_tuples) - 1
+    print negated
+    print
+
+    print "--------"
+    print "Test of triples as object."
+
+    triples_p_a = s.triples((None, URIRef("p"), triples_a), ordering="asc")
+    print "Should be 2 triples (in order):", len(triples_p_a) == 2
+    print triples_p_a
+    print
+    print "Should have had 6 triples before:", len(triples_a) == 6
+    print triples_a
+    print
+
     s.remove((URIRef("c"), URIRef("p"), URIRef("a")))
     s.remove((URIRef("c"), URIRef("q"), URIRef("a")))
     s.remove((URIRef("d"), URIRef("p"), URIRef("a")))
     s.remove((URIRef("d"), URIRef("p"), URIRef("b")))
     subject_a = s.subject(URIRef("a"))
-    # Test of triple combination.
-    result = s.triples([t2, subject_a])
+
+    print "--------"
+    print "Test of triple combination."
+
+    result = s.triples(expression=s.and_triples([t2, subject_a]))
     print "Should be 3 triples:", len(result) == 3
     print result
     print
-    # Test of triple combination using expressions.
-    result = s.triples(s.Expression("_ or _", [t2, subject_a]))
+
+    print "--------"
+    print "Test of triple combination using expressions."
+
+    result = s.triples(expression=s.or_triples([t2, subject_a]))
     print "Should be 9 triples:", len(result) == 9
     print result
     print
-    # Test of triple combination.
-    t1_and_t2 = s.count([t1, t2])
+
+    print "--------"
+    print "Test of triple combination."
+
+    t1_and_t2 = s.count(expression=s.and_triples([t1, t2]))
     print "Should be 2:", len(t1_and_t2) == 2
     print len(t1_and_t2)
     print
@@ -236,7 +347,7 @@ if not args.has_key("no-test"):
     print "Should be 1 triple:", len(result) == 1
     print result
     print
-    result = s.triples((None, None, s2))
+    result = s.triples((None, None, subjects_with_object_gt_2))
     print "Should be 1 triple:", len(result) == 1
     print result
     print
